@@ -2,8 +2,9 @@ from typing import Set, Dict, List, Tuple, Any, Optional
 from collections import defaultdict
 
 from ailment.block import Block
-from ailment.statement import Statement, Assignment, Store, Call
-from ailment.expression import Register, Convert, Load, StackBaseOffset
+from ailment.statement import Statement, Assignment, Store, Call, ConditionalJump
+from ailment.expression import Register, Convert, Load, StackBaseOffset, Expression, DirtyExpression, \
+    VEXCCallExpression
 
 from ...engines.light import SpOffset
 from ...knowledge_plugins.key_definitions.constants import OP_AFTER
@@ -17,10 +18,13 @@ from .. import Analysis, AnalysesHub
 from .ailblock_walker import AILBlockWalker
 from .ailgraph_walker import AILGraphWalker
 from .block_simplifier import BlockSimplifier
+from .ccall_rewriters import CCALL_REWRITERS
 
 
 class HasCallNotification(Exception):
-    pass
+    """
+    Notifies the existence of a call statement.
+    """
 
 
 class AILSimplifier(Analysis):
@@ -54,7 +58,12 @@ class AILSimplifier(Analysis):
         self.simplified |= folded_exprs
         if folded_exprs:
             self._rebuild_func_graph()
-            # reading definition analysis results are no longer reliable
+            # reaching definition analysis results are no longer reliable
+            return
+
+        ccalls_rewritten = self._rewrite_ccalls()
+        if ccalls_rewritten:
+            self._rebuild_func_graph()
             return
 
         if self._unify_vars:
@@ -226,14 +235,17 @@ class AILSimplifier(Analysis):
 
                 if defs and len(defs) == 1:
                     stackvar_def = defs[0]
-                    if isinstance(stackvar_def.atom, atoms.MemoryLocation) and isinstance(stackvar_def.atom.addr, SpOffset):
+                    if isinstance(stackvar_def.atom, atoms.MemoryLocation) \
+                            and isinstance(stackvar_def.atom.addr, SpOffset):
                         # found the stack variable
                         # Make sure there is no other write to this location
-                        if any((def_ != stackvar_def and def_.atom == stackvar_def.atom) for def_ in rd.all_definitions if isinstance(def_.atom, atoms.MemoryLocation)):
+                        if any((def_ != stackvar_def and def_.atom == stackvar_def.atom)
+                               for def_ in rd.all_definitions if isinstance(def_.atom, atoms.MemoryLocation)):
                             continue
 
                         # Make sure the register is never updated across this function
-                        if any((def_ != the_def and def_.atom == the_def.atom) for def_ in rd.all_definitions if isinstance(def_.atom, atoms.Register)):
+                        if any((def_ != the_def and def_.atom == the_def.atom)
+                               for def_ in rd.all_definitions if isinstance(def_.atom, atoms.Register)):
                             continue
 
                         # find all its uses
@@ -242,8 +254,8 @@ class AILSimplifier(Analysis):
                         for use in all_stackvar_uses:
                             all_uses_with_def.add((stackvar_def, use))
 
-                        to_replace = Load(None, StackBaseOffset(None, self.project.arch.bits, eq.atom0.offset), eq.atom0.size,
-                                          endness=self.project.arch.memory_endness)
+                        to_replace = Load(None, StackBaseOffset(None, self.project.arch.bits, eq.atom0.offset),
+                                          eq.atom0.size, endness=self.project.arch.memory_endness)
                         replace_with = eq.atom1
                         remove_initial_assignment = True
 
@@ -260,11 +272,15 @@ class AILSimplifier(Analysis):
                 if isinstance(eq.atom0, SimStackVariable):
                     # create the memory loading expression
                     to_replace = eq.atom1
-                    replace_with = Load(None, StackBaseOffset(None, self.project.arch.bits, eq.atom0.offset), eq.atom0.size,
-                               endness=self.project.arch.memory_endness)
+                    replace_with = Load(None, StackBaseOffset(None, self.project.arch.bits, eq.atom0.offset),
+                                        eq.atom0.size, endness=self.project.arch.memory_endness)
                 elif isinstance(eq.atom0, Register):
-                    to_replace = eq.atom1
-                    replace_with = eq.atom0
+                    if self.project.arch.is_artificial_register(eq.atom0.reg_offset, eq.atom0.size):
+                        to_replace = eq.atom0
+                        replace_with = eq.atom1
+                    else:
+                        to_replace = eq.atom1
+                        replace_with = eq.atom0
                 else:
                     raise RuntimeError("Unsupported atom0 type %s." % type(eq.atom0))
 
@@ -308,11 +324,18 @@ class AILSimplifier(Analysis):
 
         Example::
 
-            s0 = func();
+            s1 = func();
+            s0 = s1;
             if (s0) ...
 
         after folding, it will be transformed to::
 
+            s0 = func();
+            if (s0) ...
+
+        to avoid cases where func() is called more than once after simplification, another simplification pass will run
+        on the structured graph to further transform it to::
+
             if (func()) ...
         """
 
@@ -355,6 +378,11 @@ class AILSimplifier(Analysis):
                     continue
                 u = next(iter(all_uses))
 
+                # check the statement and make sure it's not a conditional jump
+                the_block = addr_and_idx_to_block[(u.block_addr, u.block_idx)]
+                if isinstance(the_block.statements[u.stmt_idx], ConditionalJump):
+                    continue
+
                 # check if the use and the definition is within the same supernode
                 super_node_blocks = self._get_super_node_blocks(
                     addr_and_idx_to_block[(the_def.codeloc.block_addr, the_def.codeloc.block_idx)]
@@ -522,6 +550,48 @@ class AILSimplifier(Analysis):
 
         return simplified
 
+    def _rewrite_ccalls(self):
+        rewriter_cls = CCALL_REWRITERS.get(self.project.arch.name, None)
+        if rewriter_cls is None:
+            return False
+
+        walker = None
+
+        class _any_update:
+            """
+            Dummy class for storing if any result has been updated.
+            """
+            v = False
+
+        def _handle_expr(expr_idx: int, expr: Expression, stmt_idx: int,
+                         stmt: Statement, block) -> Optional[Expression]:
+
+            if isinstance(expr, DirtyExpression) and isinstance(expr.dirty_expr, VEXCCallExpression):
+                rewriter = rewriter_cls(expr.dirty_expr)
+                if rewriter.result is not None:
+                    _any_update.v = True
+                    return rewriter.result
+                else:
+                    return None
+
+            return AILBlockWalker._handle_expr(walker, expr_idx, expr, stmt_idx, stmt, block)
+
+        blocks_by_addr_and_idx = dict(((node.addr, node.idx), node) for node in self.func_graph.nodes())
+
+        walker = AILBlockWalker()
+        walker._handle_expr = _handle_expr
+
+        updated = False
+        for block in blocks_by_addr_and_idx.values():
+            _any_update.v = False
+            old_block = block.copy()
+            walker.walk(block)
+            if _any_update.v:
+                self.blocks[old_block] = block
+                updated = True
+
+        return updated
+
     @staticmethod
     def _statement_has_call_exprs(stmt: Statement) -> bool:
 
