@@ -1,4 +1,5 @@
 from typing import Optional, List, Tuple, Any, Set, TYPE_CHECKING
+import copy
 import logging
 
 import archinfo
@@ -6,8 +7,8 @@ from ailment import Stmt, Expr
 
 from ...procedures.stubs.format_parser import FormatParser, FormatSpecifier
 from ...errors import SimMemoryMissingError
-from ...sim_type import SimTypeBottom, SimTypePointer, SimTypeChar
-from ...calling_conventions import SimRegArg, SimStackArg
+from ...sim_type import SimTypeBottom, SimTypePointer, SimTypeChar, SimTypeInt
+from ...calling_conventions import SimRegArg, SimStackArg, SimCC
 from ...knowledge_plugins.key_definitions.constants import OP_BEFORE
 from ...knowledge_plugins.key_definitions.definition import Definition
 from .. import Analysis, register_analysis
@@ -65,30 +66,28 @@ class CallSiteMaker(Analysis):
 
             args = [ ]
             arg_locs = None
-            if func.calling_convention is None:
+            if cc is None:
                 l.warning('%s has an unknown calling convention.', repr(func))
             else:
                 stackarg_sp_diff = func.calling_convention.STACKARG_SP_DIFF
-                if func.prototype is not None:
+                if prototype is not None:
                     # Make arguments
-                    arg_locs = func.calling_convention.arg_locs()
-                    if func.prototype.variadic:
+                    arg_locs = cc.arg_locs(prototype)
+                    if prototype.variadic:
                         # determine the number of variadic arguments
-                        variadic_args = self._determine_variadic_arguments(func, func.calling_convention, last_stmt)
+                        variadic_args = self._determine_variadic_arguments(func, cc, last_stmt)
                         if variadic_args:
-                            arg_sizes = [arg.size // self.project.arch.byte_width for arg in func.prototype.args] + \
-                                        ([self.project.arch.bytes] * variadic_args)
-                            is_fp = [False] * len(arg_sizes)
-                            arg_locs = func.calling_convention.arg_locs(is_fp=is_fp, sizes=arg_sizes)
-                else:
-                    if func.calling_convention.args is not None:
-                        arg_locs = func.calling_convention.arg_locs()
+                            callsite_ty = copy.copy(prototype)
+                            callsite_ty.args = list(callsite_ty.args)
+                            for i in range(variadic_args):
+                                callsite_ty.args.append(SimTypeInt().with_arch(self.project.arch))
+                            arg_locs = cc.arg_locs(callsite_ty)
 
             if arg_locs is not None:
                 for arg_loc in arg_locs:
                     if type(arg_loc) is SimRegArg:
                         size = arg_loc.size
-                        offset = arg_loc._fix_offset(None, size, arch=self.project.arch)
+                        offset = arg_loc.check_offset(cc.arch)
 
                         _, the_arg = self._resolve_register_argument(last_stmt, arg_loc)
 
@@ -194,7 +193,7 @@ class CallSiteMaker(Analysis):
     def _resolve_register_argument(self, call_stmt, arg_loc) -> Tuple:
 
         size = arg_loc.size
-        offset = arg_loc._fix_offset(None, size, arch=self.project.arch)
+        offset = arg_loc.check_offset(self.project.arch)
 
         if self._reaching_definitions is not None:
             # Find its definition
@@ -310,9 +309,7 @@ class CallSiteMaker(Analysis):
 
         fmt_str = None
         min_arg_count = (max(potential_fmt_args) + 1)
-        arg_locs = cc.arg_locs(is_fp=[False] * min_arg_count,
-                               sizes=[self.project.arch.bytes] * min_arg_count
-                               )
+        arg_locs = cc.arg_locs(SimCC.guess_prototype([0]*min_arg_count, proto))
 
         for fmt_arg_idx in potential_fmt_args:
             arg_loc = arg_locs[fmt_arg_idx]
