@@ -18,6 +18,7 @@ from ...knowledge_plugins.key_definitions.live_definitions import Definition
 from .subject import SubjectType
 from .external_codeloc import ExternalCodeLocation
 from .rd_state import ReachingDefinitionsState
+from .function_handler import FunctionHandler
 
 l = logging.getLogger(name=__name__)
 
@@ -30,7 +31,8 @@ class SimEngineRDAIL(
     arch: archinfo.Arch
     state: ReachingDefinitionsState
 
-    def __init__(self, project, call_stack, maximum_local_call_depth, function_handler=None):
+    def __init__(self, project, call_stack, maximum_local_call_depth,
+                 function_handler: Optional[FunctionHandler] = None):
         super().__init__()
         self.project = project
         self._call_stack = call_stack
@@ -143,20 +145,13 @@ class SimEngineRDAIL(
 
     def _ail_handle_ConditionalJump(self, stmt):
 
-        cond = self._expr(stmt.condition)  # pylint:disable=unused-variable
-        true_target = self._expr(stmt.true_target)  # pylint:disable=unused-variable
-        false_target = self._expr(stmt.false_target)  # pylint:disable=unused-variable
+        _ = self._expr(stmt.condition)  # pylint:disable=unused-variable
+        _ = self._expr(stmt.true_target)  # pylint:disable=unused-variable
+        _ = self._expr(stmt.false_target)  # pylint:disable=unused-variable
 
         ip = Register(self.arch.ip_offset, self.arch.bytes)
         self.state.kill_definitions(ip)
 
-        # kill all cc_ops
-        if 'cc_op' in self.arch.registers:
-            self.state.kill_definitions(Register(*self.arch.registers['cc_op']))
-            self.state.kill_definitions(Register(*self.arch.registers['cc_dep1']))
-            self.state.kill_definitions(Register(*self.arch.registers['cc_dep2']))
-            self.state.kill_definitions(Register(*self.arch.registers['cc_ndep']))
-
     def _ail_handle_Call(self, stmt: ailment.Stmt.Call):
         self._handle_Call_base(stmt, is_expr=False)
 
@@ -172,11 +167,10 @@ class SimEngineRDAIL(
         if stmt.args is not None:
             # getting used expressions from stmt.args
             used_exprs = stmt.args
-        elif stmt.calling_convention is not None and (
-                stmt.calling_convention.func_ty is not None or stmt.calling_convention.args is not None):
+        elif stmt.calling_convention is not None and stmt.prototype is not None:
             # getting used expressions from the function prototype, its arguments, and the calling convention
             used_exprs = [ ]
-            for arg_loc in stmt.calling_convention.arg_locs():
+            for arg_loc in stmt.calling_convention.arg_locs(stmt.prototype):
                 if isinstance(arg_loc, SimRegArg):
                     used_exprs.append(Register(self.arch.registers[arg_loc.reg_name][0], arg_loc.size))
                 elif isinstance(arg_loc, SimStackArg):
@@ -243,11 +237,12 @@ class SimEngineRDAIL(
     def _ail_handle_Return(self, stmt: ailment.Stmt.Return):  # pylint:disable=unused-argument
 
         codeloc = self._codeloc()
-        size = self.project.arch.bits // 8
 
         cc = None
+        prototype = None
         if self.state.analysis.subject.type == SubjectType.Function:
             cc = self.state.analysis.subject.content.calling_convention
+            prototype = self.state.analysis.subject.content.prototype
             # import ipdb; ipdb.set_trace()
 
         if cc is None:
@@ -280,9 +275,14 @@ class SimEngineRDAIL(
         # consume registers that are potentially useful
 
         # return value
-        if cc is not None and cc.ret_val is not None:
-            if isinstance(cc.ret_val, SimRegArg):
-                offset = cc.ret_val._fix_offset(None, size, arch=self.project.arch)
+        if cc is not None and prototype is not None and prototype.returnty is not None:
+            ret_val = cc.return_val(prototype.returnty)
+            if isinstance(ret_val, SimRegArg):
+                if ret_val.clear_entire_reg:
+                    offset, size = cc.arch.registers[ret_val.reg_name]
+                else:
+                    offset = cc.arch.registers[ret_val.reg_name][0] + ret_val.reg_offset
+                    size = ret_val.size
                 self.state.add_use(Register(offset, size), codeloc)
             else:
                 l.error("Cannot handle CC with non-register return value location")
@@ -575,13 +575,15 @@ class SimEngineRDAIL(
             # each value in expr0 >> expr1_v
             if len(expr0.values) == 1 and 0 in expr0.values:
                 if all(v.concrete for v in expr0.values[0]):
-                    vs = {(claripy.LShR(v, expr1_v._model_concrete.value) if v.concrete else self.state.top(bits)) for v in expr0.values[0]}
+                    vs = {(claripy.LShR(v, expr1_v._model_concrete.value) if v.concrete else self.state.top(bits))
+                          for v in expr0.values[0]}
                     r = MultiValues(offset_to_values={0: vs})
         elif expr0_v is not None and expr1_v is None:
             # expr0_v >> each value in expr1
             if len(expr1.values) == 1 and 0 in expr1.values:
                 if all(v.concrete for v in expr1.values[0]):
-                    vs = {(claripy.LShR(expr0_v, v._model_concrete.value) if v.concrete else self.state.top(bits)) for v in expr1.values[0]}
+                    vs = {(claripy.LShR(expr0_v, v._model_concrete.value) if v.concrete else self.state.top(bits))
+                          for v in expr1.values[0]}
                     r = MultiValues(offset_to_values={0: vs})
         else:
             if expr0_v.concrete and expr1_v.concrete:
@@ -607,13 +609,15 @@ class SimEngineRDAIL(
             # each value in expr0 >> expr1_v
             if len(expr0.values) == 1 and 0 in expr0.values:
                 if all(v.concrete for v in expr0.values[0]):
-                    vs = {(claripy.LShR(v, expr1_v._model_concrete.value) if v.concrete else self.state.top(bits)) for v in expr0.values[0]}
+                    vs = {(claripy.LShR(v, expr1_v._model_concrete.value) if v.concrete else self.state.top(bits))
+                          for v in expr0.values[0]}
                     r = MultiValues(offset_to_values={0: vs})
         elif expr0_v is not None and expr1_v is None:
             # expr0_v >> each value in expr1
             if len(expr1.values) == 1 and 0 in expr1.values:
                 if all(v.concrete for v in expr1.values[0]):
-                    vs = {(claripy.LShR(expr0_v, v._model_concrete.value) if v.concrete else self.state.top(bits)) for v in expr1.values[0]}
+                    vs = {(claripy.LShR(expr0_v, v._model_concrete.value) if v.concrete else self.state.top(bits))
+                          for v in expr1.values[0]}
                     r = MultiValues(offset_to_values={0: vs})
         else:
             if expr0_v.concrete and expr1_v.concrete:
@@ -639,13 +643,15 @@ class SimEngineRDAIL(
             # each value in expr0 << expr1_v
             if len(expr0.values) == 1 and 0 in expr0.values:
                 if all(v.concrete for v in expr0.values[0]):
-                    vs = {((v << expr1_v._model_concrete.value) if v.concrete else self.state.top(bits)) for v in expr0.values[0]}
+                    vs = {((v << expr1_v._model_concrete.value) if v.concrete else self.state.top(bits))
+                          for v in expr0.values[0]}
                     r = MultiValues(offset_to_values={0: vs})
         elif expr0_v is not None and expr1_v is None:
             # expr0_v >> each value in expr1
             if len(expr1.values) == 1 and 0 in expr1.values:
                 if all(v.concrete for v in expr1.values[0]):
-                    vs = {((expr0_v << v._model_concrete.value) if v.concrete else self.state.top(bits)) for v in expr1.values[0]}
+                    vs = {((expr0_v << v._model_concrete.value) if v.concrete else self.state.top(bits))
+                          for v in expr1.values[0]}
                     r = MultiValues(offset_to_values={0: vs})
         else:
             if expr0_v.concrete and expr1_v.concrete:
@@ -828,8 +834,14 @@ class SimEngineRDAIL(
         stack_addr = self.state.stack_address(expr.offset)
         return MultiValues(offset_to_values={0: {stack_addr}})
 
-    def _ail_handle_DirtyExpression(self, expr: ailment.Expr.DirtyExpression) -> MultiValues:  # pylint:disable=no-self-use
-        # FIXME: DirtyExpression needs .bits
+    def _ail_handle_DirtyExpression(self,
+                                    expr: ailment.Expr.DirtyExpression
+                                    ) -> MultiValues:  # pylint:disable=no-self-use
+
+        if isinstance(expr.dirty_expr, ailment.Expr.VEXCCallExpression):
+            for operand in expr.dirty_expr.operands:
+                self._expr(operand)
+
         top = self.state.top(expr.bits)
         return MultiValues(offset_to_values={0: {top}})
 
@@ -858,41 +870,32 @@ class SimEngineRDAIL(
             return None
 
         is_internal = False
-        ext_func_name = None
+        ext_func_name: Optional[str] = None
+        symbol = None
         if self.project.loader.main_object.contains_addr(ip_addr) is True:
             ext_func_name = self.project.loader.find_plt_stub_name(ip_addr)
             if ext_func_name is None:
                 is_internal = True
         else:
             symbol = self.project.loader.find_symbol(ip_addr)
-            if symbol is not None:
-                ext_func_name = symbol.name
-
-        if ext_func_name is not None:
-            handler_name = 'handle_%s' % ext_func_name
-            if hasattr(self._function_handler, handler_name):
-                getattr(self._function_handler, handler_name)(self.state, self._codeloc())
-            else:
-                l.warning('Please implement the external function handler for %s() with your own logic.',
-                          ext_func_name)
+        if symbol is not None:
+            self._function_handler.handle_external_function_symbol(self.state, symbol, self._codeloc())
+        elif ext_func_name is not None:
+            self._function_handler.handle_external_function_name(self.state, ext_func_name, self._codeloc())
         elif is_internal is True:
-            handler_name = 'handle_local_function'
-            if hasattr(self._function_handler, handler_name):
-                is_updated, state, visited_blocks, dep_graph = getattr(self._function_handler, handler_name)(
-                    self.state,
-                    ip_addr,
-                    self._call_stack,
-                    self._maximum_local_call_depth,
-                    self._visited_blocks,
-                    self._dep_graph,
-                )
+            is_updated, state, visited_blocks, dep_graph = self._function_handler.handle_local_function(
+                self.state,
+                ip_addr,
+                self._call_stack,
+                self._maximum_local_call_depth,
+                self._visited_blocks,
+                self._dep_graph,
+            )
 
-                if is_updated is True:
-                    self.state = state
-                    self._visited_blocks = visited_blocks
-                    self._dep_graph = dep_graph
-            else:
-                l.warning('Please implement the local function handler with your own logic.')
+            if is_updated is True:
+                self.state = state
+                self._visited_blocks = visited_blocks
+                self._dep_graph = dep_graph
         else:
             l.warning('Could not find function name for external function at address %#x.', ip_addr)
         return None
