@@ -1,22 +1,22 @@
+# pylint:disable=wrong-import-position,arguments-differ
 import logging
 from typing import List
 
+import pyvex
 from pyvex import IRSB
+from archinfo import ArchARM
 
 try:
     from .engines import pcode
 except ImportError:
     pcode = None
 
-l = logging.getLogger(name=__name__)
-
-import pyvex
-from archinfo import ArchARM
-
 from .protos import primitives_pb2 as pb2
 from .serializable import Serializable
 from .engines.vex import VEXLifter
 
+l = logging.getLogger(name=__name__)
+
 DEFAULT_VEX_ENGINE = VEXLifter(None)  # this is only used when Block is not initialized with a project
 
 
@@ -49,6 +49,8 @@ class DisassemblerInsn:
     Helper class to represent a disassembled target architecture instruction
     """
 
+    __slots__ = ()
+
     @property
     def size(self) -> int:
         raise NotImplementedError()
@@ -78,8 +80,15 @@ class CapstoneBlock(DisassemblerBlock):
     outside of capstone itself
     """
 
+    __slots__ = ()
+
 
 class CapstoneInsn(DisassemblerInsn):
+    """
+    Represents a capstone instruction.
+    """
+
+    __slots__ = ("insn", )
 
     def __init__(self, capstone_insn):
         self.insn = capstone_insn
@@ -109,17 +118,21 @@ class CapstoneInsn(DisassemblerInsn):
 
 
 class Block(Serializable):
+    """
+    Represents a basic block in a binary or a program.
+    """
+
     BLOCK_MAX_SIZE = 4096
 
     __slots__ = ['_project', '_bytes', '_vex', 'thumb', '_disassembly', '_capstone',
                  'addr', 'size', 'arch', '_instructions', '_instruction_addrs',
                  '_opt_level', '_vex_nostmt', '_collect_data_refs', '_strict_block_end',
-                 '_cross_insn_opt',
+                 '_cross_insn_opt', '_load_from_ro_regions',
                  ]
 
-    def __init__(self, addr, project=None, arch=None, size=None, byte_string=None, vex=None, thumb=False, backup_state=None,
-                 extra_stop_points=None, opt_level=None, num_inst=None, traceflags=0, strict_block_end=None,
-                 collect_data_refs=False, cross_insn_opt=True):
+    def __init__(self, addr, project=None, arch=None, size=None, byte_string=None, vex=None, thumb=False,
+                 backup_state=None, extra_stop_points=None, opt_level=None, num_inst=None, traceflags=0,
+                 strict_block_end=None, collect_data_refs=False, cross_insn_opt=True, load_from_ro_regions=False):
 
         # set up arch
         if project is not None:
@@ -164,6 +177,7 @@ class Block(Serializable):
                         traceflags=traceflags,
                         strict_block_end=strict_block_end,
                         collect_data_refs=collect_data_refs,
+                        load_from_ro_regions=load_from_ro_regions,
                         cross_insn_opt=cross_insn_opt,
                 )
                 size = vex.size
@@ -176,11 +190,12 @@ class Block(Serializable):
         self._collect_data_refs = collect_data_refs
         self._strict_block_end = strict_block_end
         self._cross_insn_opt = cross_insn_opt
+        self._load_from_ro_regions = load_from_ro_regions
 
         self._instructions = num_inst
         self._instruction_addrs = [] # type: List[int]
 
-        self._parse_vex_info()
+        self._parse_vex_info(self._vex)
 
         if byte_string is None:
             if backup_state is not None:
@@ -202,19 +217,11 @@ class Block(Serializable):
             # size will ALWAYS be known at this point
             self._bytes = str(pyvex.ffi.buffer(byte_string, self.size))
 
-    def _parse_vex_info(self):
-        vex = self._vex
-        if vex is not None:
-            self._instructions = vex.instructions
-            self._instruction_addrs = []
-            self.size = vex.size
-
-            for stmt in vex.statements:
-                if stmt.tag != 'Ist_IMark':
-                    continue
-                if self.addr is None:
-                    self.addr = stmt.addr + stmt.delta
-                self._instruction_addrs.append(stmt.addr + stmt.delta)
+    def _parse_vex_info(self, vex_block):
+        if vex_block is not None:
+            self._instructions = vex_block.instructions
+            self._instruction_addrs = vex_block.instruction_addresses
+            self.size = vex_block.size
 
     def __repr__(self):
         return '<Block for %#x, %d bytes>' % (self.addr, self.size)
@@ -267,8 +274,9 @@ class Block(Serializable):
                     collect_data_refs=self._collect_data_refs,
                     strict_block_end=self._strict_block_end,
                     cross_insn_opt=self._cross_insn_opt,
+                    load_from_ro_regions=self._load_from_ro_regions,
             )
-            self._parse_vex_info()
+            self._parse_vex_info(self._vex)
 
         return self._vex
 
@@ -293,7 +301,9 @@ class Block(Serializable):
             collect_data_refs=self._collect_data_refs,
             strict_block_end=self._strict_block_end,
             cross_insn_opt=self._cross_insn_opt,
+            load_from_ro_regions=self._load_from_ro_regions,
         )
+        self._parse_vex_info(self._vex_nostmt)
         return self._vex_nostmt
 
     @property
@@ -303,7 +313,7 @@ class Block(Serializable):
     @property
     def disassembly(self) -> DisassemblerBlock:
         """
-        Provide a disassebly object using whatever disassembler is available
+        Provide a disassembly object using whatever disassembler is available
         """
         if self._disassembly is None:
             if self._using_pcode_engine:
@@ -314,13 +324,17 @@ class Block(Serializable):
 
     @property
     def capstone(self):
-        if self._capstone: return self._capstone
+        if self._capstone:
+            return self._capstone
 
         cs = self.arch.capstone if not self.thumb else self.arch.capstone_thumb
 
         insns = []
 
-        for cs_insn in cs.disasm(self.bytes, self.addr):
+        block_bytes = self.bytes
+        if self.size is not None:
+            block_bytes = block_bytes[:self.size]
+        for cs_insn in cs.disasm(block_bytes, self.addr):
             insns.append(CapstoneInsn(cs_insn))
         block = CapstoneBlock(self.addr, insns, self.thumb, self.arch)
 
@@ -378,6 +392,10 @@ class Block(Serializable):
 
 
 class SootBlock:
+    """
+    Represents a Soot IR basic block.
+    """
+
     def __init__(self, addr, project=None, arch=None):
 
         self.addr = addr
@@ -389,8 +407,7 @@ class SootBlock:
     def _soot_engine(self):
         if self._project is None:
             raise Exception('SHIIIIIIIT')
-        else:
-            return self._project.factory.default_engine
+        return self._project.factory.default_engine
 
     @property
     def soot(self):
@@ -408,4 +425,5 @@ class SootBlock:
         stmts_len = len(stmts) if stmts else 0
         return SootBlockNode(self.addr, stmts_len, stmts=stmts)
 
+
 from .codenode import BlockNode, SootBlockNode
