@@ -18,7 +18,7 @@ from ..engine import SimEngine
 
 class SimEngineLightMixin:
     def __init__(self, *args, logger=None, **kwargs):
-        self.arch: archinfo.Arch = None
+        self.arch: Optional[archinfo.Arch] = None
         self.l = logger
         super().__init__(*args, **kwargs)
 
@@ -127,6 +127,7 @@ class SimEngineLight(
                             )
 
 
+# noinspection PyPep8Naming
 class SimEngineLightVEXMixin(SimEngineLightMixin):
 
     def _process(self, state, successors, *args, block, whitelist=None, **kwargs):  # pylint:disable=arguments-differ
@@ -233,6 +234,10 @@ class SimEngineLightVEXMixin(SimEngineLightMixin):
             self.l.error('Unsupported expression type %s.', type(expr).__name__)
         return None
 
+    def _handle_Triop(self, expr: pyvex.IRExpr.Triop):
+        self.l.error('Unsupported Triop %s.', expr.op)
+        return None
+
     def _handle_RdTmp(self, expr):
         tmp = expr.tmp
 
@@ -280,10 +285,10 @@ class SimEngineLightVEXMixin(SimEngineLightMixin):
             handler = '_handle_Not1'
         elif expr.op.startswith('Iop_Not'):
             handler = '_handle_Not'
-        elif expr.op.startswith('Iop_Clz64'):
-            handler = '_handle_Clz64'
-        elif expr.op.startswith('Iop_Ctz64'):
-            handler = '_handle_Ctz64'
+        elif expr.op.startswith('Iop_Clz'):
+            handler = '_handle_Clz'
+        elif expr.op.startswith('Iop_Ctz'):
+            handler = '_handle_Ctz'
 
         if handler is not None and hasattr(self, handler):
             return getattr(self, handler)(expr)
@@ -291,7 +296,7 @@ class SimEngineLightVEXMixin(SimEngineLightMixin):
             self.l.error('Unsupported Unop %s.', expr.op)
             return None
 
-    def _handle_Binop(self, expr):
+    def _handle_Binop(self, expr: pyvex.IRExpr.Binop):
         handler = None
         if expr.op.startswith('Iop_And'):
             handler = '_handle_And'
@@ -301,8 +306,14 @@ class SimEngineLightVEXMixin(SimEngineLightMixin):
             handler = '_handle_Or'
         elif expr.op.startswith('Iop_Add'):
             handler = '_handle_Add'
+        elif expr.op.startswith('Iop_HAdd'):
+            handler = '_handle_HAdd'
         elif expr.op.startswith('Iop_Sub'):
             handler = '_handle_Sub'
+        elif expr.op.startswith('Iop_QSub'):
+            handler = '_handle_QSub'
+        elif expr.op.startswith('Iop_Mull'):
+            handler = "_handle_Mull"
         elif expr.op.startswith('Iop_Mul'):
             handler = "_handle_Mul"
         elif expr.op.startswith('Iop_DivMod'):
@@ -334,6 +345,8 @@ class SimEngineLightVEXMixin(SimEngineLightMixin):
             handler = '_handle_CmpGT'
         elif expr.op.startswith('Iop_CmpORD'):
             handler = '_handle_CmpORD'
+        elif expr.op.startswith('Iop_CmpF'):
+            handler = '_handle_CmpF'
         elif expr.op == 'Iop_32HLto64':
             handler = '_handle_32HLto64'
         elif expr.op.startswith('Const'):
@@ -346,7 +359,7 @@ class SimEngineLightVEXMixin(SimEngineLightMixin):
         vector_size, vector_count = None, None
         if handler is not None:
             # vector information
-            m = re.match(r"Iop_[^\d]+(\d+)x(\d+)", expr.op)
+            m = re.match(r"Iop_[^\d]+(\d+)U{0,1}x(\d+)", expr.op)
             if m is not None:
                 vector_size = int(m.group(1))
                 vector_count = int(m.group(2))
@@ -464,7 +477,7 @@ class SimEngineLightVEXMixin(SimEngineLightMixin):
             self.l.exception(e)
             return None
 
-    def _handle_Clz64(self, expr):
+    def _handle_Clz(self, expr):
         arg0 = expr.args[0]
         expr_0 = self._expr(arg0)
         if expr_0 is None:
@@ -473,7 +486,7 @@ class SimEngineLightVEXMixin(SimEngineLightMixin):
             return self._top(expr_0.size())
         return self._top(expr_0.size())
 
-    def _handle_Ctz64(self, expr):
+    def _handle_Ctz(self, expr):
         arg0 = expr.args[0]
         expr_0 = self._expr(arg0)
         if expr_0 is None:
@@ -703,31 +716,31 @@ class SimEngineLightVEXMixin(SimEngineLightMixin):
 
         return expr_0 > expr_1
 
-    def _handle_CmpEQ_v(self, expr, vector_size, vector_count):
+    def _handle_CmpEQ_v(self, expr, _vector_size, _vector_count):
         _, _ = self._binop_get_args(expr)
         return self._top(expr.result_size(self.tyenv))
 
-    def _handle_CmpNE_v(self, expr, vector_size, vector_count):
+    def _handle_CmpNE_v(self, expr, _vector_size, _vector_count):
         _, _ = self._binop_get_args(expr)
         return self._top(expr.result_size(self.tyenv))
 
-    def _handle_CmpLE_v(self, expr, vector_size, vector_count):
+    def _handle_CmpLE_v(self, expr, _vector_size, _vector_count):
         _, _ = self._binop_get_args(expr)
         return self._top(expr.result_size(self.tyenv))
 
-    def _handle_CmpGE_v(self, expr, vector_size, vector_count):
+    def _handle_CmpGE_v(self, expr, _vector_size, _vector_count):
         _, _ = self._binop_get_args(expr)
         return self._top(expr.result_size(self.tyenv))
 
-    def _handle_CmpLT_v(self, expr, vector_size, vector_count):
+    def _handle_CmpLT_v(self, expr, _vector_size, _vector_count):
         _, _ = self._binop_get_args(expr)
         return self._top(expr.result_size(self.tyenv))
 
-    def _handle_CmpGT_v(self, expr, vector_size, vector_count):
+    def _handle_CmpGT_v(self, expr, _vector_size, _vector_count):
         _, _ = self._binop_get_args(expr)
         return self._top(expr.result_size(self.tyenv))
 
-    def _handle_MBE(self, expr):
+    def _handle_MBE(self, _expr: pyvex.IRStmt.MBE):
         # Yeah.... no.
         return None
 
@@ -751,6 +764,7 @@ class SimEngineLightVEXMixin(SimEngineLightMixin):
         return self._top(expr.result_size(self.tyenv))
 
 
+# noinspection PyPep8Naming
 class SimEngineLightAILMixin(SimEngineLightMixin):
 
     def _process(self, state, successors, *args, block=None, whitelist=None, **kwargs):  # pylint:disable=arguments-differ
@@ -807,7 +821,9 @@ class SimEngineLightAILMixin(SimEngineLightMixin):
     #
 
     def _codeloc(self):
-        return CodeLocation(self.block.addr, self.stmt_idx, ins_addr=self.ins_addr, context=self._context,
+        # noinspection PyUnresolvedReferences
+        return CodeLocation(self.block.addr, self.stmt_idx, ins_addr=self.ins_addr,
+                            context=self._context,
                             block_idx=self.block.idx)
 
     #
@@ -864,7 +880,10 @@ class SimEngineLightAILMixin(SimEngineLightMixin):
     def _ail_handle_Reinterpret(self, expr: ailment.Expr.Reinterpret):
         arg = self._expr(expr.operand)
 
-        if isinstance(arg, int) and expr.from_bits == 32 and expr.from_type == "I" and expr.to_bits == 32 and expr.to_type == "F":
+        if isinstance(arg, int) and (expr.from_bits == 32
+                                     and expr.from_type == "I"
+                                     and expr.to_bits == 32
+                                     and expr.to_type == "F"):
             # int -> float
             b = struct.pack("<I", arg)
             f = struct.unpack("<f", b)[0]
