@@ -1,8 +1,9 @@
 # pylint:disable=no-member
 import pickle
 import logging
-from typing import Optional, List, Dict
+from typing import Optional, List, Dict, Tuple
 from collections import defaultdict
+import bisect
 
 import networkx
 
@@ -25,12 +26,13 @@ class CFGModel(Serializable):
     """
 
     __slots__ = ('ident', 'graph', 'jump_tables', 'memory_data', 'insn_addr_to_memory_data', '_nodes_by_addr',
-                 '_nodes', '_cfg_manager', '_iropt_level', )
+                 '_nodes', '_cfg_manager', '_iropt_level', '_node_addrs', 'is_arm')
 
-    def __init__(self, ident, cfg_manager=None):
+    def __init__(self, ident, cfg_manager=None, is_arm=False):
 
         self.ident = ident
         self._cfg_manager = cfg_manager
+        self.is_arm = is_arm
 
         # Necessary settings
         self._iropt_level = None
@@ -43,14 +45,16 @@ class CFGModel(Serializable):
 
         # Memory references
         # A mapping between address and the actual data in memory
-        self.memory_data = { }
+        self.memory_data: Dict[int,MemoryData] = { }
         # A mapping between address of the instruction that's referencing the memory data and the memory data itself
-        self.insn_addr_to_memory_data = { }
+        self.insn_addr_to_memory_data: Dict[int,MemoryData] = { }
 
         # Lists of CFGNodes indexed by the address of each block. Don't serialize
         self._nodes_by_addr = defaultdict(list)
         # CFGNodes dict indexed by block ID. Don't serialize
         self._nodes = { }
+        # addresses of CFGNodes to speed up get_any_node(..., anyaddr=True). Don't serialize
+        self._node_addrs = [ ]
 
     #
     # Properties
@@ -109,7 +113,7 @@ class CFGModel(Serializable):
                 if k == 'jumpkind':
                     edge.jumpkind = cfg_jumpkind_to_pb(v)
                 elif k == 'ins_addr':
-                    edge.ins_addr = v if v is not None else -1
+                    edge.ins_addr = v if v is not None else 0xffff_ffff_ffff_ffff
                 elif k == 'stmt_idx':
                     edge.stmt_idx = v if v is not None else -1
                 else:
@@ -144,6 +148,8 @@ class CFGModel(Serializable):
                     l.warning("Importing a CFG with more than one node for a given address is currently unsupported. "
                               "The resulting graph may be broken.")
 
+        model._node_addrs = list(sorted(model._nodes_by_addr.keys()))
+
         # edges
         for edge_pb2 in cmsg.edges:
             # more than one node at a given address is unsupported, grab the first one
@@ -153,7 +159,7 @@ class CFGModel(Serializable):
             for k, v in edge_pb2.data.items():
                 data[k] = pickle.loads(v)
             data['jumpkind'] = cfg_jumpkind_from_pb(edge_pb2.jumpkind)
-            data['ins_addr'] = edge_pb2.ins_addr if edge_pb2.ins_addr != -1 else None
+            data['ins_addr'] = edge_pb2.ins_addr if edge_pb2.ins_addr != 0xffff_ffff_ffff_ffff else None
             data['stmt_idx'] = edge_pb2.stmt_idx if edge_pb2.stmt_idx != -1 else None
             model.graph.add_edge(src, dst, **data)
 
@@ -172,7 +178,7 @@ class CFGModel(Serializable):
     #
 
     def copy(self):
-        model = CFGModel(self.ident, cfg_manager=self._cfg_manager)
+        model = CFGModel(self.ident, cfg_manager=self._cfg_manager, is_arm=self.is_arm)
         model.graph = networkx.DiGraph(self.graph)
         model.jump_tables = self.jump_tables.copy()
         model.memory_data = self.memory_data.copy()
@@ -182,6 +188,35 @@ class CFGModel(Serializable):
 
         return model
 
+    #
+    # Node insertion and removal
+    #
+
+    def add_node(self, block_id: int, node: CFGNode) -> None:
+        self._nodes[block_id] = node
+        self._nodes_by_addr[node.addr].append(node)
+
+        if isinstance(node.addr, int):
+            pos = bisect.bisect_left(self._node_addrs, node.addr)
+            if pos >= len(self._node_addrs):
+                self._node_addrs.append(node.addr)
+            elif self._node_addrs[pos] != node.addr:
+                self._node_addrs.insert(pos, node.addr)
+
+    def remove_node(self, block_id: int, node: CFGNode) -> None:
+        if block_id in self._nodes:
+            del self._nodes[block_id]
+
+        if node.addr in self._nodes_by_addr:
+            self._nodes_by_addr[node.addr].remove(node)
+            if not self._nodes_by_addr[node.addr]:
+                del self._nodes_by_addr[node.addr]
+
+                if isinstance(node.addr, int):
+                    pos = bisect.bisect_left(self._node_addrs, node.addr)
+                    if pos < len(self._node_addrs) and self._node_addrs[pos] == node.addr:
+                        self._node_addrs.pop(pos)
+
     #
     # CFG View
     #
@@ -211,16 +246,14 @@ class CFGModel(Serializable):
                                 isn't a syscall node.
         :param anyaddr:         If anyaddr is True, then addr doesn't have to be the beginning address of a basic
                                 block. By default the entire graph.nodes() will be iterated, and the first node
-                                containing the specific address is returned, which is slow. If you need to do many such
-                                queries, you may first call `generate_index()` to create some indices that may speed up
-                                the query.
+                                containing the specific address is returned, which can be slow.
         :param force_fastpath:  If force_fastpath is True, it will only perform a dict lookup in the _nodes_by_addr
                                 dict.
         :return:                A CFGNode if there is any that satisfies given conditions, or None otherwise
         """
 
         # fastpath: directly look in the nodes list
-        if not anyaddr:
+        if not anyaddr or addr in self._nodes_by_addr:
             try:
                 return self._nodes_by_addr[addr][0]
             except (KeyError, IndexError):
@@ -229,33 +262,34 @@ class CFGModel(Serializable):
         if force_fastpath:
             return None
 
-        # slower path
-        #if self._node_lookup_index is not None:
-        #    pass
+        if isinstance(addr, int):
+            # slower path
+            # find all potential addresses that the block may cover
+            pos = bisect.bisect_left(self._node_addrs, max(addr - 400, 0))
 
-        # the slowest path
-        # try to show a warning first
-        # TODO: re-enable it once the segment tree is implemented
-        #if self._node_lookup_index_warned == False:
-        #    l.warning('Calling get_any_node() with anyaddr=True is slow on large programs. '
-        #              'For better performance, you may first call generate_index() to generate some indices that may '
-        #              'speed the node lookup.')
-        #    self._node_lookup_index_warned = True
+            is_cfgemulated = self.ident == "CFGEmulated"
 
-        for n in self.graph.nodes():
-            if self.ident == "CFGEmulated":
-                cond = n.looping_times == 0
-            else:
-                cond = True
-            if anyaddr and n.size is not None:
-                cond = cond and (addr == n.addr or n.addr <= addr < n.addr + n.size)
-            else:
-                cond = cond and (addr == n.addr)
-            if cond:
-                if is_syscall is None:
-                    return n
-                if n.is_syscall == is_syscall:
-                    return n
+            while pos < len(self._node_addrs):
+                n = self._nodes_by_addr[self._node_addrs[pos]][0]
+                actual_addr = n.addr if not self.is_arm else n.addr & 0xffff_fffe
+                if actual_addr > addr:
+                    break
+
+                if is_cfgemulated:
+                    cond = n.looping_times == 0
+                else:
+                    cond = True
+                if anyaddr and n.size is not None:
+                    cond = cond and (addr == actual_addr or actual_addr <= addr < actual_addr + n.size)
+                else:
+                    cond = cond and (addr == actual_addr)
+                if cond:
+                    if is_syscall is None:
+                        return n
+                    if n.is_syscall == is_syscall:
+                        return n
+
+                pos += 1
 
         return None
 
@@ -274,11 +308,7 @@ class CFGModel(Serializable):
                                          cfg_node.size is not None and
                                          cfg_node.addr <= addr < (cfg_node.addr + cfg_node.size)
                                          ):
-                if is_syscall and cfg_node.is_syscall:
-                    results.append(cfg_node)
-                elif is_syscall is False and not cfg_node.is_syscall:
-                    results.append(cfg_node)
-                else:
+                if is_syscall is None or is_syscall == cfg_node.is_syscall:
                     results.append(cfg_node)
 
         return results
@@ -328,7 +358,10 @@ class CFGModel(Serializable):
                 predecessors.append(pred)
         return predecessors
 
-    def get_successors(self, node, excluding_fakeret=True, jumpkind=None):
+    def get_successors(self, node: CFGNode,
+                       excluding_fakeret: bool = True,
+                       jumpkind: Optional[str] = None
+                       ) -> List[CFGNode]:
         """
         Get successors of a node in the control flow graph.
 
@@ -364,7 +397,7 @@ class CFGModel(Serializable):
                 successors.append(suc)
         return successors
 
-    def get_successors_and_jumpkind(self, node, excluding_fakeret=True):
+    def get_successors_and_jumpkinds(self, node, excluding_fakeret=True):
         """
         Get a list of tuples where the first element is the successor of the CFG node and the second element is the
         jumpkind of the successor.
@@ -381,6 +414,26 @@ class CFGModel(Serializable):
                 successors.append((suc, data['jumpkind']))
         return successors
 
+    get_successors_and_jumpkind = get_successors_and_jumpkinds
+
+    def get_predecessors_and_jumpkinds(self, node: CFGNode, excluding_fakeret: bool=True) -> List[Tuple[CFGNode,str]]:
+        """
+        Get a list of tuples where the first element is the predecessor of the CFG node and the second element is the
+        jumpkind of the predecessor.
+
+        :param node:                The node.
+        :param excluding_fakeret:   True if you want to exclude all predecessors that are fall-through predecessors.
+        :return:                    A list of predecessors and their corresponding jumpkinds.
+        """
+
+        predecessors = []
+        for pred, _, data in self.graph.in_edges([node], data=True):
+            if not excluding_fakeret or data['jumpkind'] != 'Ijk_FakeRet':
+                predecessors.append((pred, data['jumpkind']))
+        return predecessors
+
+    get_predecessors_and_jumpkind = get_predecessors_and_jumpkinds
+
     def get_all_predecessors(self, cfgnode, depth_limit=None):
         """
         Get all predecessors of a specific node on the control flow graph.
